System.register("core/i/ISerializedLogRecord", [], function(exports_1, context_1) {
    "use strict";
    var __moduleName = context_1 && context_1.id;
    return {
        setters:[],
        execute: function() {
        }
    }
});
System.register("core/i/ILogRecord", [], function(exports_2, context_2) {
    "use strict";
    var __moduleName = context_2 && context_2.id;
    return {
        setters:[],
        execute: function() {
        }
    }
});
System.register("core/i/ILogHandler", [], function(exports_3, context_3) {
    "use strict";
    var __moduleName = context_3 && context_3.id;
    return {
        setters:[],
        execute: function() {
        }
    }
});
System.register("core/i/ISerializedError", [], function(exports_4, context_4) {
    "use strict";
    var __moduleName = context_4 && context_4.id;
    return {
        setters:[],
        execute: function() {
        }
    }
});
System.register("core/i/ILoggerInterfaceOptions", [], function(exports_5, context_5) {
    "use strict";
    var __moduleName = context_5 && context_5.id;
    return {
        setters:[],
        execute: function() {
        }
    }
});
System.register("core/i/ILoggingInterface", [], function(exports_6, context_6) {
    "use strict";
    var __moduleName = context_6 && context_6.id;
    return {
        setters:[],
        execute: function() {
        }
    }
});
System.register("core/i/ILogger", [], function(exports_7, context_7) {
    "use strict";
    var __moduleName = context_7 && context_7.id;
    return {
        setters:[],
        execute: function() {
        }
    }
});
System.register("core/Logger", [], function(exports_8, context_8) {
    "use strict";
    var __moduleName = context_8 && context_8.id;
    var Logger;
    return {
        setters:[],
        execute: function() {
            /**
             * Provides log functionality
             */
            Logger = (function () {
                function Logger(logRecordFactory, LoggingInterface) {
                    var _this = this;
                    this._handlers = [];
                    /**
                     * Default logging interface
                     * @type {LoggingIntersface}
                     * @protected
                     */
                    this._loggingInterface = new LoggingInterface(logRecordFactory, { location: '', tags: [] }, function (record) { return _this.save(record); });
                }
                Object.defineProperty(Logger.prototype, "handlers", {
                    /**
                     * Get list of Log Handlers names registered in logger
                     * @returns {string[]}
                     */
                    get: function () {
                        return this._handlers.map(function (h) { return h.name; });
                    },
                    enumerable: true,
                    configurable: true
                });
                /**
                 * Process log record
                 * @param {LogRecord} logRecord
                 */
                Logger.prototype.save = function (logRecord) {
                    this._handlers.forEach(function (handler) { return handler.handle(logRecord); });
                    logRecord.erase();
                };
                /**
                 * Get interface that inherits tag and location from current one
                 */
                Logger.prototype.getInterface = function (loc, tags) {
                    if (loc === void 0) { loc = ''; }
                    if (tags === void 0) { tags = []; }
                    return this._loggingInterface.getInterface(loc, tags);
                };
                /**
                 * Register new log handler
                 */
                Logger.prototype.registerHandler = function (handler) {
                    this._handlers.push(handler);
                    return this;
                };
                /**
                 * Remove registered handler by it's name
                 * @param name
                 */
                Logger.prototype.removeHandler = function (name) {
                    var _this = this;
                    this._handlers.map(function (h, i) { return h.name === name ? i : null; })
                        .filter(function (i) { return i !== null; })
                        .forEach(function (i) { return _this._handlers.splice(i, 1); });
                };
                return Logger;
            }());
            exports_8("Logger", Logger);
        }
    }
});
System.register("core/i/IError", [], function(exports_9, context_9) {
    "use strict";
    var __moduleName = context_9 && context_9.id;
    return {
        setters:[],
        execute: function() {
        }
    }
});
System.register("core/utils", [], function(exports_10, context_10) {
    "use strict";
    var __moduleName = context_10 && context_10.id;
    var LOG_EMERGENCY, LOG_ALERT, LOG_CRITICAL, LOG_ERROR, LOG_WARNING, LOG_NOTICE, LOG_INFORMATIONAL, LOG_DEBUG, LOG_INFO;
    /**
     * Merge locations
     */
    function joinLocation(base, addition) {
        if (!addition) {
            return base;
        }
        return [base, addition].join('.');
    }
    exports_10("joinLocation", joinLocation);
    /**
     * Modify base array adding all tag from addition
     */
    function mergeTags(base, addition) {
        Array.prototype.push.apply(base, addition);
        return base;
    }
    exports_10("mergeTags", mergeTags);
    /**
     * Get label by syslog level tag
     */
    function logLevelLabel(logLevel) {
        switch (logLevel) {
            case LOG_EMERGENCY: return 'emergency';
            case LOG_ALERT: return 'alert';
            case LOG_CRITICAL: return 'critical';
            case LOG_ERROR: return 'error';
            case LOG_WARNING: return 'warning';
            case LOG_NOTICE: return 'notice';
            case LOG_INFORMATIONAL: return 'info';
            case LOG_DEBUG: return 'debug';
            default: return '';
        }
    }
    exports_10("logLevelLabel", logLevelLabel);
    /**
     * Check if list of tags has log level from searchLogLevels array
     * Returns first met tag or empty string if log level not found
     */
    function hasLogLevel(tags, searchLogLevels) {
        var logLevel = tags.filter(function (tag) { return searchLogLevels.indexOf(tag) !== -1; });
        return logLevel.length ? logLevel[0] : '';
    }
    exports_10("hasLogLevel", hasLogLevel);
    /**
     * Convert error object to simple object with error information
     * Returns first met tag or empty string if log level not found
     */
    function serializeError(error) {
        return {
            type: error.name,
            msg: error.message,
            stack: error.stack
        };
    }
    exports_10("serializeError", serializeError);
    return {
        setters:[],
        execute: function() {
            // Syslog levels: https://en.wikipedia.org/wiki/Syslog
            //_______________________________________________________________________________________________________
            //| Value |   Severity	   |  Keyword  |	Description	                                                 |
            //|=======|================|===========|=================================================================|
            exports_10("LOG_EMERGENCY", LOG_EMERGENCY = '__emerg'); //|  0    | Emergency      | emerg     | System is unusable                                              |
            exports_10("LOG_ALERT", LOG_ALERT = '__alert'); //|  1    | Alert          | alert     | Should be corrected immediately                                 |
            exports_10("LOG_CRITICAL", LOG_CRITICAL = '__crit'); //|  2    | Critical       | crit      | Critical conditions                                             |
            exports_10("LOG_ERROR", LOG_ERROR = '__err'); //|  3    | Error          | err       | Error conditions                                                |
            exports_10("LOG_WARNING", LOG_WARNING = '__warning'); //|  4    | Warning        | warning   | May indicate that an error will occur if action is not taken    |
            exports_10("LOG_NOTICE", LOG_NOTICE = '__notice'); //|  5    | Notice         | notice    | Events that are unusual, but not error conditions               |
            exports_10("LOG_INFORMATIONAL", LOG_INFORMATIONAL = '__info'); //|  6    | Informational  | info      | Normal operational messages that require no action              |
            exports_10("LOG_DEBUG", LOG_DEBUG = '__debug'); //|  7    | Debug          | debug     | Information useful to developers for debugging the application  |
            //|_______|________________|___________|_________________________________________________________________|
            exports_10("LOG_INFO", LOG_INFO = LOG_INFORMATIONAL); // Alias for LOG_INFORMATIONAL
        }
    }
});
System.register("core/LogRecord", ["core/utils"], function(exports_11, context_11) {
    "use strict";
    var __moduleName = context_11 && context_11.id;
    var utils_1;
    var LogRecord;
    return {
        setters:[
            function (utils_1_1) {
                utils_1 = utils_1_1;
            }],
        execute: function() {
            //TODO: implement uid
            LogRecord = (function () {
                function LogRecord(tags, loc, message, saveLogFunc) {
                    this._dt = new Date();
                    this._tags = tags;
                    this._location = loc;
                    this._saveLogFunc = saveLogFunc;
                    this._message = message;
                    this._data = null;
                }
                LogRecord.create = function (tags, loc, message, saveLogFunc) {
                    return new LogRecord(tags, loc, message, saveLogFunc);
                };
                Object.defineProperty(LogRecord.prototype, "tags", {
                    /**
                     * Get list of tags, should return copy of array of tags not the reference
                     */
                    get: function () {
                        return this._tags.slice();
                    },
                    enumerable: true,
                    configurable: true
                });
                /**
                 * Attach data related to the log record
                 */
                LogRecord.prototype.data = function (value) {
                    this._data = value;
                    return this;
                };
                /**
                 * Attach new tags to log record
                 */
                LogRecord.prototype.tag = function (value) {
                    utils_1.mergeTags(this._tags, value);
                    return this;
                };
                /**
                 * Specify location of a log record
                 */
                LogRecord.prototype.loc = function (value) {
                    this._location = utils_1.joinLocation(this._location, value);
                    return this;
                };
                /**
                 * Add Emergency tag
                 */
                LogRecord.prototype.emerg = function () {
                    utils_1.mergeTags(this._tags, [utils_1.LOG_EMERGENCY]);
                    return this;
                };
                /**
                 * Add Alert tag
                 */
                LogRecord.prototype.alert = function () {
                    utils_1.mergeTags(this._tags, [utils_1.LOG_ALERT]);
                    return this;
                };
                /**
                 * Add Critical tag
                 */
                LogRecord.prototype.crit = function () {
                    utils_1.mergeTags(this._tags, [utils_1.LOG_CRITICAL]);
                    return this;
                };
                /**
                 * Add Error tag
                 */
                LogRecord.prototype.error = function () {
                    utils_1.mergeTags(this._tags, [utils_1.LOG_ERROR]);
                    return this;
                };
                /**
                 * Add Warning tag
                 */
                LogRecord.prototype.warning = function () {
                    utils_1.mergeTags(this._tags, [utils_1.LOG_WARNING]);
                    return this;
                };
                /**
                 * Add Notice tag
                 */
                LogRecord.prototype.notice = function () {
                    utils_1.mergeTags(this._tags, [utils_1.LOG_NOTICE]);
                    return this;
                };
                /**
                 * Add Info tag
                 */
                LogRecord.prototype.info = function () {
                    utils_1.mergeTags(this._tags, [utils_1.LOG_INFORMATIONAL]);
                    return this;
                };
                /**
                 * Add Debug tag
                 */
                LogRecord.prototype.debug = function () {
                    utils_1.mergeTags(this._tags, [utils_1.LOG_DEBUG]);
                    return this;
                };
                /**
                 * Serialize log record to object
                 */
                LogRecord.prototype.toObject = function () {
                    //TODO: implement uid
                    return {
                        dt: this._dt,
                        //uid: this._uid,
                        message: this._message,
                        data: this._data,
                        tags: this._tags,
                        location: this._location
                    };
                };
                /**
                 * Process log record
                 */
                LogRecord.prototype.log = function () {
                    this._saveLogFunc(this);
                };
                /**
                 * Erase all data of the log record, to prevent memory leaks
                 */
                LogRecord.prototype.erase = function () {
                    this._dt = null;
                    this._tags = null;
                    this._location = null;
                    this._saveLogFunc = null;
                    this._message = null;
                    this._data = null;
                    //TODO: implement uid
                    //this._uid = null;
                };
                return LogRecord;
            }());
            exports_11("LogRecord", LogRecord);
        }
    }
});
System.register("core/LoggingInterface", ["core/utils"], function(exports_12, context_12) {
    "use strict";
    var __moduleName = context_12 && context_12.id;
    var utils_2;
    var LoggingInterface;
    return {
        setters:[
            function (utils_2_1) {
                utils_2 = utils_2_1;
            }],
        execute: function() {
            /**
             * Provides logging interface
             */
            LoggingInterface = (function () {
                function LoggingInterface(logRecordFactory, _a, saveFunc) {
                    var _b = _a.tags, tags = _b === void 0 ? [] : _b, _c = _a.location, location = _c === void 0 ? '' : _c;
                    this._saveFunc = saveFunc;
                    this._tags = tags;
                    this._location = location;
                    this._logRecordFactory = logRecordFactory;
                }
                Object.defineProperty(LoggingInterface.prototype, "tags", {
                    get: function () {
                        return this._tags.slice();
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(LoggingInterface.prototype, "location", {
                    get: function () {
                        return this._location;
                    },
                    enumerable: true,
                    configurable: true
                });
                /**
                 * Get new log record
                 * @param message Text message that should be logged
                 * @returns Returns instance of log record, that can be modified before processing
                 */
                LoggingInterface.prototype.l = function (message) {
                    return this._logRecordFactory.create(this.tags, this.location, message, this._saveFunc);
                };
                /**
                 * Creates new Log record and process it
                 * @param message Text message that describes log record
                 * @param data Data attached to the log record
                 * @param loc Location, where this log done
                 * @param tags List of tags attached for this specific log record
                 */
                LoggingInterface.prototype.log = function (message, data, loc, tags) {
                    if (data === void 0) { data = null; }
                    if (loc === void 0) { loc = null; }
                    if (tags === void 0) { tags = []; }
                    this.l(message).data(data).loc(loc).tag(tags).log();
                };
                /**
                 * Creates new Log record with Emergency tag and process it
                 * @param message Text message that describes log record
                 * @param data Data attached to the log record
                 * @param loc Location, where this log done
                 * @param tags List of tags attached for this specific log record
                 */
                LoggingInterface.prototype.emerg = function (message, data, loc, tags) {
                    if (data === void 0) { data = null; }
                    if (loc === void 0) { loc = null; }
                    if (tags === void 0) { tags = []; }
                    this.log(message, data, loc, utils_2.mergeTags(tags, [utils_2.LOG_EMERGENCY]));
                };
                /**
                 * Creates new Log record with Alert tag and process it
                 * @param message Text message that describes log record
                 * @param data Data attached to the log record
                 * @param loc Location, where this log done
                 * @param tags List of tags attached for this specific log record
                 */
                LoggingInterface.prototype.alert = function (message, data, loc, tags) {
                    if (data === void 0) { data = null; }
                    if (loc === void 0) { loc = null; }
                    if (tags === void 0) { tags = []; }
                    this.log(message, data, loc, utils_2.mergeTags(tags, [utils_2.LOG_ALERT]));
                };
                /**
                 * Creates new Log record with Critical tag and process it
                 * @param message Text message that describes log record
                 * @param data Data attached to the log record
                 * @param loc Location, where this log done
                 * @param tags List of tags attached for this specific log record
                 */
                LoggingInterface.prototype.crit = function (message, data, loc, tags) {
                    if (data === void 0) { data = null; }
                    if (loc === void 0) { loc = null; }
                    if (tags === void 0) { tags = []; }
                    this.log(message, data, loc, utils_2.mergeTags(tags, [utils_2.LOG_CRITICAL]));
                };
                /**
                 * Creates new Log record with Error tag and process it
                 * @param message Text message that describes log record
                 * @param data Data attached to the log record
                 * @param loc Location, where this log done
                 * @param tags List of tags attached for this specific log record
                 */
                LoggingInterface.prototype.error = function (message, data, loc, tags) {
                    if (data === void 0) { data = null; }
                    if (loc === void 0) { loc = null; }
                    if (tags === void 0) { tags = []; }
                    this.log(message, data, loc, utils_2.mergeTags(tags, [utils_2.LOG_ERROR]));
                };
                /**
                 * Creates new Log record with Warning tag and process it
                 * @param message Text message that describes log record
                 * @param data Data attached to the log record
                 * @param loc Location, where this log done
                 * @param tags List of tags attached for this specific log record
                 */
                LoggingInterface.prototype.warning = function (message, data, loc, tags) {
                    if (data === void 0) { data = null; }
                    if (loc === void 0) { loc = null; }
                    if (tags === void 0) { tags = []; }
                    this.log(message, data, loc, utils_2.mergeTags(tags, [utils_2.LOG_WARNING]));
                };
                /**
                 * Creates new Log record with Notice tag and process it
                 * @param message Text message that describes log record
                 * @param data Data attached to the log record
                 * @param loc Location, where this log done
                 * @param tags List of tags attached for this specific log record
                 */
                LoggingInterface.prototype.notice = function (message, data, loc, tags) {
                    if (data === void 0) { data = null; }
                    if (loc === void 0) { loc = null; }
                    if (tags === void 0) { tags = []; }
                    this.log(message, data, loc, utils_2.mergeTags(tags, [utils_2.LOG_NOTICE]));
                };
                /**
                 * Creates new Log record with Info tag and process it
                 * @param message Text message that describes log record
                 * @param data Data attached to the log record
                 * @param loc Location, where this log done
                 * @param tags List of tags attached for this specific log record
                 */
                LoggingInterface.prototype.info = function (message, data, loc, tags) {
                    if (data === void 0) { data = null; }
                    if (loc === void 0) { loc = null; }
                    if (tags === void 0) { tags = []; }
                    this.log(message, data, loc, utils_2.mergeTags(tags, [utils_2.LOG_INFORMATIONAL]));
                };
                /**
                 * Creates new Log record with Debug tag and process it
                 * @param message Text message that describes log record
                 * @param data Data attached to the log record
                 * @param loc Location, where this log done
                 * @param tags List of tags attached for this specific log record
                 */
                LoggingInterface.prototype.debug = function (message, data, loc, tags) {
                    if (data === void 0) { data = null; }
                    if (loc === void 0) { loc = null; }
                    if (tags === void 0) { tags = []; }
                    this.log(message, data, loc, utils_2.mergeTags(tags, [utils_2.LOG_DEBUG]));
                };
                /**
                 * Serialize error to object
                 */
                LoggingInterface.prototype.serializeError = function (error) {
                    return utils_2.serializeError(error);
                };
                /**
                 * Get new logging interface that inherits tag and location
                 * @param loc Location that will be added to location of the current logging interface
                 * @param tags List of tags that will be merged with tags of the current logging interface
                 */
                LoggingInterface.prototype.getInterface = function (loc, tags) {
                    return new LoggingInterface(this._logRecordFactory, {
                        tags: utils_2.mergeTags(this._tags.slice(), tags),
                        location: utils_2.joinLocation(this._location, loc)
                    }, this._saveFunc);
                };
                return LoggingInterface;
            }());
            exports_12("LoggingInterface", LoggingInterface);
        }
    }
});
System.register("core/i/ILogFilter", [], function(exports_13, context_13) {
    "use strict";
    var __moduleName = context_13 && context_13.id;
    return {
        setters:[],
        execute: function() {
        }
    }
});
System.register("core/i/ILogFormatter", [], function(exports_14, context_14) {
    "use strict";
    var __moduleName = context_14 && context_14.id;
    return {
        setters:[],
        execute: function() {
        }
    }
});
System.register("core/i/ILogWriter", [], function(exports_15, context_15) {
    "use strict";
    var __moduleName = context_15 && context_15.id;
    return {
        setters:[],
        execute: function() {
        }
    }
});
System.register("core/LogHandler", [], function(exports_16, context_16) {
    "use strict";
    var __moduleName = context_16 && context_16.id;
    var LogHandler;
    return {
        setters:[],
        execute: function() {
            /**
             * Stores pair of filter and writer and coordinate them to handle log records
             */
            LogHandler = (function () {
                function LogHandler(name, filter, formatter, writer) {
                    this._name = name;
                    this._filter = filter;
                    this._formatter = formatter;
                    this._writer = writer;
                }
                Object.defineProperty(LogHandler.prototype, "name", {
                    get: function () {
                        return this._name;
                    },
                    enumerable: true,
                    configurable: true
                });
                /**
                 * Process log record, if it match filter than pass it to writer
                 */
                LogHandler.prototype.handle = function (logRecord) {
                    var _this = this;
                    if (!this._filter.match(logRecord)) {
                        return;
                    }
                    var logData = logRecord.toObject();
                    setTimeout(function () {
                        _this._writer.save(_this._formatter.format(logData));
                    }, 0);
                };
                return LogHandler;
            }());
            exports_16("LogHandler", LogHandler);
        }
    }
});
System.register("core/i/ILoggerFactory", [], function(exports_17, context_17) {
    "use strict";
    var __moduleName = context_17 && context_17.id;
    return {
        setters:[],
        execute: function() {
        }
    }
});
System.register("core/index", ["core/Logger", "core/LoggingInterface", "core/LogRecord", "core/LogHandler"], function(exports_18, context_18) {
    "use strict";
    var __moduleName = context_18 && context_18.id;
    var Logger_1, LoggingInterface_1, LogRecord_1, LogHandler_1;
    return {
        setters:[
            function (Logger_1_1) {
                Logger_1 = Logger_1_1;
            },
            function (LoggingInterface_1_1) {
                LoggingInterface_1 = LoggingInterface_1_1;
            },
            function (LogRecord_1_1) {
                LogRecord_1 = LogRecord_1_1;
            },
            function (LogHandler_1_1) {
                LogHandler_1 = LogHandler_1_1;
            }],
        execute: function() {
            exports_18("Logger", Logger_1.Logger);
            exports_18("LoggingInterface", LoggingInterface_1.LoggingInterface);
            exports_18("LogRecord", LogRecord_1.LogRecord);
            exports_18("LogHandler", LogHandler_1.LogHandler);
        }
    }
});
System.register("filters/FilterAny", [], function(exports_19, context_19) {
    "use strict";
    var __moduleName = context_19 && context_19.id;
    var FilterAny;
    return {
        setters:[],
        execute: function() {
            /**
             * Filter that pass all log records
             *
             */
            FilterAny = (function () {
                function FilterAny() {
                }
                Object.defineProperty(FilterAny, "entityName", {
                    // We need it for es5 compilation. Because in es5 we dont have classes and we can't just access FilterAny.name
                    get: function () {
                        return 'FilterAny';
                    },
                    enumerable: true,
                    configurable: true
                });
                FilterAny.create = function () {
                    return new FilterAny();
                };
                /**
                 * Check if log record match current filter
                 */
                FilterAny.prototype.match = function (logRecord) {
                    return true;
                };
                return FilterAny;
            }());
            exports_19("FilterAny", FilterAny);
        }
    }
});
System.register("formatters/FormatterSingleLine", ["core/utils"], function(exports_20, context_20) {
    "use strict";
    var __moduleName = context_20 && context_20.id;
    var utils_3;
    var FormatterSingleLine;
    return {
        setters:[
            function (utils_3_1) {
                utils_3 = utils_3_1;
            }],
        execute: function() {
            FormatterSingleLine = (function () {
                function FormatterSingleLine() {
                }
                Object.defineProperty(FormatterSingleLine, "entityName", {
                    // We need it for es5 compilation. Because in es5 we dont have classes and we can't just access FormatterSingleLine.name
                    get: function () {
                        return 'FormatterSingleLine';
                    },
                    enumerable: true,
                    configurable: true
                });
                FormatterSingleLine.create = function () {
                    return new FormatterSingleLine();
                };
                FormatterSingleLine.prototype.format = function (logData) {
                    var result = [];
                    result.push("[" + logData.dt.toLocaleString() + "]");
                    result.push(logData.message);
                    result.push(logData.location);
                    var attachedData = logData.data;
                    if (attachedData instanceof Error) {
                        attachedData = utils_3.serializeError(attachedData);
                    }
                    if (typeof attachedData === 'object' && attachedData !== null) {
                        if ('toObject' in attachedData) {
                            attachedData = JSON.stringify(attachedData.toObject());
                        }
                        else {
                            attachedData = JSON.stringify(attachedData);
                        }
                    }
                    result.push(attachedData);
                    result.push(logData.tags);
                    result.push('\n');
                    return result.join('\t');
                };
                return FormatterSingleLine;
            }());
            exports_20("FormatterSingleLine", FormatterSingleLine);
        }
    }
});
System.register("writers/WriterToConsole", [], function(exports_21, context_21) {
    "use strict";
    var __moduleName = context_21 && context_21.id;
    var WriterToConsole;
    return {
        setters:[],
        execute: function() {
            /**
             * Writes logs to console
             */
            WriterToConsole = (function () {
                function WriterToConsole() {
                }
                Object.defineProperty(WriterToConsole, "entityName", {
                    // We need it for es5 compilation. Because in es5 we dont have classes and we can't just access WriterToConsole.name
                    get: function () {
                        return 'WriterToConsole';
                    },
                    enumerable: true,
                    configurable: true
                });
                WriterToConsole.create = function () {
                    return new WriterToConsole();
                };
                /**
                 * Save log record
                 */
                WriterToConsole.prototype.save = function (item) {
                    console.log(item);
                };
                return WriterToConsole;
            }());
            exports_21("WriterToConsole", WriterToConsole);
        }
    }
});
System.register("core/LoggerFactory", ["core/index"], function(exports_22, context_22) {
    "use strict";
    var __moduleName = context_22 && context_22.id;
    var index_1;
    var LoggerFactory;
    return {
        setters:[
            function (index_1_1) {
                index_1 = index_1_1;
            }],
        execute: function() {
            LoggerFactory = (function () {
                function LoggerFactory() {
                    this._filters = {};
                    this._formatters = {};
                    this._writers = {};
                }
                LoggerFactory.prototype.applyLogHandler = function (l, o) {
                    if (!this._filters[o.filter]) {
                        throw new Error("Unknown filter '" + o.filter + "'");
                    }
                    if (!this._formatters[o.formatter]) {
                        throw new Error("Unknown formatter '" + o.formatter + "'");
                    }
                    if (!this._writers[o.writer]) {
                        throw new Error("Unknown writer '" + o.writer + "'");
                    }
                    var FilterFactory = this._filters[o.filter];
                    var Formatter = this._formatters[o.formatter];
                    var Writer = this._writers[o.writer];
                    var logHandler = new index_1.LogHandler(o.name, FilterFactory.create(), Formatter.create(), Writer.create());
                    l.registerHandler(logHandler);
                };
                LoggerFactory.prototype.createEmptyLogger = function () {
                    return new index_1.Logger(index_1.LogRecord, index_1.LoggingInterface);
                };
                LoggerFactory.prototype.createDefaultLogger = function () {
                    var l = new index_1.Logger(index_1.LogRecord, index_1.LoggingInterface);
                    this.applyLogHandler(l, {
                        name: 'any_to_console_as_single_line',
                        filter: 'FilterAny',
                        formatter: 'FormatterSingleLine',
                        writer: 'WriterToConsole'
                    });
                    return l;
                };
                LoggerFactory.prototype.createLogger = function (logHandlers) {
                    var _this = this;
                    var l = new index_1.Logger(index_1.LogRecord, index_1.LoggingInterface);
                    logHandlers.forEach(function (handlerOptions) { return _this.applyLogHandler(l, handlerOptions); });
                    return l;
                };
                LoggerFactory.prototype.registerFilter = function (entity) {
                    this._filters[entity.entityName] = entity;
                };
                LoggerFactory.prototype.registerFormatter = function (entity) {
                    this._formatters[entity.entityName] = entity;
                };
                LoggerFactory.prototype.registerWriter = function (entity) {
                    this._writers[entity.entityName] = entity;
                };
                return LoggerFactory;
            }());
            exports_22("LoggerFactory", LoggerFactory);
        }
    }
});
System.register("ntc-logger", ["core/Logger", "core/LogRecord", "core/LoggingInterface", "core/LogHandler", "core/LoggerFactory", "filters/FilterAny", "formatters/FormatterSingleLine", "writers/WriterToConsole"], function(exports_23, context_23) {
    "use strict";
    var __moduleName = context_23 && context_23.id;
    var Logger_2, LogRecord_2, LoggingInterface_2, LogHandler_2, LoggerFactory_1, FilterAny_1, FormatterSingleLine_1, WriterToConsole_1;
    var NtcLogger;
    return {
        setters:[
            function (Logger_2_1) {
                Logger_2 = Logger_2_1;
            },
            function (LogRecord_2_1) {
                LogRecord_2 = LogRecord_2_1;
            },
            function (LoggingInterface_2_1) {
                LoggingInterface_2 = LoggingInterface_2_1;
            },
            function (LogHandler_2_1) {
                LogHandler_2 = LogHandler_2_1;
            },
            function (LoggerFactory_1_1) {
                LoggerFactory_1 = LoggerFactory_1_1;
            },
            function (FilterAny_1_1) {
                FilterAny_1 = FilterAny_1_1;
            },
            function (FormatterSingleLine_1_1) {
                FormatterSingleLine_1 = FormatterSingleLine_1_1;
            },
            function (WriterToConsole_1_1) {
                WriterToConsole_1 = WriterToConsole_1_1;
            }],
        execute: function() {
            NtcLogger = new LoggerFactory_1.LoggerFactory();
            NtcLogger.registerFilter(FilterAny_1.FilterAny);
            NtcLogger.registerFormatter(FormatterSingleLine_1.FormatterSingleLine);
            NtcLogger.registerWriter(WriterToConsole_1.WriterToConsole);
            exports_23("NtcLogger", NtcLogger);
            exports_23("LoggerFactory", LoggerFactory_1.LoggerFactory);
            exports_23("Logger", Logger_2.Logger);
            exports_23("LogRecord", LogRecord_2.LogRecord);
            exports_23("LoggingInterface", LoggingInterface_2.LoggingInterface);
            exports_23("LogHandler", LogHandler_2.LogHandler);
            exports_23("FilterAny", FilterAny_1.FilterAny);
            exports_23("FormatterSingleLine", FormatterSingleLine_1.FormatterSingleLine);
            exports_23("WriterToConsole", WriterToConsole_1.WriterToConsole);
        }
    }
});
//# sourceMappingURL=js-ntc-logger.min.js.map