"use strict";
/**
 * Provides log functionality
 */
var Logger = (function () {
    function Logger(logRecordFactory, LoggingInterface) {
        var _this = this;
        this._handlers = [];
        /**
         * Default logging interface
         * @type {LoggingIntersface}
         * @protected
         */
        this._loggingInterface = new LoggingInterface(logRecordFactory, { location: '', tags: [] }, function (record) { return _this.save(record); });
    }
    Object.defineProperty(Logger.prototype, "handlers", {
        /**
         * Get list of Log Handlers names registered in logger
         * @returns {string[]}
         */
        get: function () {
            return this._handlers.map(function (h) { return h.name; });
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Process log record
     * @param {LogRecord} logRecord
     */
    Logger.prototype.save = function (logRecord) {
        this._handlers.forEach(function (handler) { return handler.handle(logRecord); });
        logRecord.erase();
    };
    /**
     * Get interface that inherits tag and location from current one
     */
    Logger.prototype.getInterface = function (loc, tags) {
        if (loc === void 0) { loc = ''; }
        if (tags === void 0) { tags = []; }
        return this._loggingInterface.getInterface(loc, tags);
    };
    /**
     * Register new log handler
     */
    Logger.prototype.registerHandler = function (handler) {
        this._handlers.push(handler);
        return this;
    };
    /**
     * Remove registered handler by it's name
     * @param name
     */
    Logger.prototype.removeHandler = function (name) {
        var _this = this;
        this._handlers.map(function (h, i) { return h.name === name ? i : null; })
            .filter(function (i) { return i !== null; })
            .forEach(function (i) { return _this._handlers.splice(i, 1); });
    };
    return Logger;
}());
exports.Logger = Logger;
