"use strict";
var index_1 = require('./index');
var LoggerFactory = (function () {
    function LoggerFactory() {
        this._filters = {};
        this._formatters = {};
        this._writers = {};
    }
    LoggerFactory.prototype.applyLogHandler = function (l, o) {
        if (!this._filters[o.filter]) {
            throw new Error("Unknown filter '" + o.filter + "'");
        }
        if (!this._formatters[o.formatter]) {
            throw new Error("Unknown formatter '" + o.formatter + "'");
        }
        if (!this._writers[o.writer]) {
            throw new Error("Unknown writer '" + o.writer + "'");
        }
        var FilterFactory = this._filters[o.filter];
        var Formatter = this._formatters[o.formatter];
        var Writer = this._writers[o.writer];
        var logHandler = new index_1.LogHandler(o.name, FilterFactory.create(), Formatter.create(), Writer.create());
        l.registerHandler(logHandler);
    };
    LoggerFactory.prototype.createEmptyLogger = function () {
        return new index_1.Logger(index_1.LogRecord, index_1.LoggingInterface);
    };
    LoggerFactory.prototype.createDefaultLogger = function () {
        var l = new index_1.Logger(index_1.LogRecord, index_1.LoggingInterface);
        this.applyLogHandler(l, {
            name: 'any_to_console_as_single_line',
            filter: 'FilterAny',
            formatter: 'FormatterSingleLine',
            writer: 'WriterToConsole'
        });
        return l;
    };
    LoggerFactory.prototype.createLogger = function (logHandlers) {
        var _this = this;
        var l = new index_1.Logger(index_1.LogRecord, index_1.LoggingInterface);
        logHandlers.forEach(function (handlerOptions) { return _this.applyLogHandler(l, handlerOptions); });
        return l;
    };
    LoggerFactory.prototype.registerFilter = function (entity) {
        this._filters[entity.entityName] = entity;
    };
    LoggerFactory.prototype.registerFormatter = function (entity) {
        this._formatters[entity.entityName] = entity;
    };
    LoggerFactory.prototype.registerWriter = function (entity) {
        this._writers[entity.entityName] = entity;
    };
    return LoggerFactory;
}());
exports.LoggerFactory = LoggerFactory;
