"use strict";
const utils_1 = require('./utils');
//TODO: implement uid
class LogRecord {
    constructor(tags, loc, message, saveLogFunc) {
        this._dt = new Date();
        this._tags = tags;
        this._location = loc;
        this._saveLogFunc = saveLogFunc;
        this._message = message;
        this._data = null;
    }
    static create(tags, loc, message, saveLogFunc) {
        return new LogRecord(tags, loc, message, saveLogFunc);
    }
    /**
     * Get list of tags, should return copy of array of tags not the reference
     */
    get tags() {
        return this._tags.slice();
    }
    /**
     * Attach data related to the log record
     */
    data(value) {
        this._data = value;
        return this;
    }
    /**
     * Attach new tags to log record
     */
    tag(value) {
        utils_1.mergeTags(this._tags, value);
        return this;
    }
    /**
     * Specify location of a log record
     */
    loc(value) {
        this._location = utils_1.joinLocation(this._location, value);
        return this;
    }
    /**
     * Add Emergency tag
     */
    emerg() {
        utils_1.mergeTags(this._tags, [utils_1.LOG_EMERGENCY]);
        return this;
    }
    /**
     * Add Alert tag
     */
    alert() {
        utils_1.mergeTags(this._tags, [utils_1.LOG_ALERT]);
        return this;
    }
    /**
     * Add Critical tag
     */
    crit() {
        utils_1.mergeTags(this._tags, [utils_1.LOG_CRITICAL]);
        return this;
    }
    /**
     * Add Error tag
     */
    error() {
        utils_1.mergeTags(this._tags, [utils_1.LOG_ERROR]);
        return this;
    }
    /**
     * Add Warning tag
     */
    warning() {
        utils_1.mergeTags(this._tags, [utils_1.LOG_WARNING]);
        return this;
    }
    /**
     * Add Notice tag
     */
    notice() {
        utils_1.mergeTags(this._tags, [utils_1.LOG_NOTICE]);
        return this;
    }
    /**
     * Add Info tag
     */
    info() {
        utils_1.mergeTags(this._tags, [utils_1.LOG_INFORMATIONAL]);
        return this;
    }
    /**
     * Add Debug tag
     */
    debug() {
        utils_1.mergeTags(this._tags, [utils_1.LOG_DEBUG]);
        return this;
    }
    /**
     * Serialize log record to object
     */
    toObject() {
        //TODO: implement uid
        return {
            dt: this._dt,
            //uid: this._uid,
            message: this._message,
            data: this._data,
            tags: this._tags,
            location: this._location
        };
    }
    /**
     * Process log record
     */
    log() {
        this._saveLogFunc(this);
    }
    /**
     * Erase all data of the log record, to prevent memory leaks
     */
    erase() {
        this._dt = null;
        this._tags = null;
        this._location = null;
        this._saveLogFunc = null;
        this._message = null;
        this._data = null;
        //TODO: implement uid
        //this._uid = null;
    }
}
exports.LogRecord = LogRecord;
