"use strict";
/**
 * Provides log functionality
 */
class Logger {
    constructor(logRecordFactory, LoggingInterface) {
        this._handlers = [];
        /**
         * Default logging interface
         * @type {LoggingIntersface}
         * @protected
         */
        this._loggingInterface = new LoggingInterface(logRecordFactory, { location: '', tags: [] }, (record) => this.save(record));
    }
    /**
     * Get list of Log Handlers names registered in logger
     * @returns {string[]}
     */
    get handlers() {
        return this._handlers.map(h => h.name);
    }
    /**
     * Process log record
     * @param {LogRecord} logRecord
     */
    save(logRecord) {
        this._handlers.forEach(handler => handler.handle(logRecord));
        logRecord.erase();
    }
    /**
     * Get interface that inherits tag and location from current one
     */
    getInterface(loc = '', tags = []) {
        return this._loggingInterface.getInterface(loc, tags);
    }
    /**
     * Register new log handler
     */
    registerHandler(handler) {
        this._handlers.push(handler);
        return this;
    }
    /**
     * Remove registered handler by it's name
     * @param name
     */
    removeHandler(name) {
        this._handlers.map((h, i) => h.name === name ? i : null)
            .filter(i => i !== null)
            .forEach(i => this._handlers.splice(i, 1));
    }
}
exports.Logger = Logger;
