"use strict";
const index_1 = require('./index');
class LoggerFactory {
    constructor() {
        this._filters = {};
        this._formatters = {};
        this._writers = {};
    }
    applyLogHandler(l, o) {
        if (!this._filters[o.filter]) {
            throw new Error(`Unknown filter '${o.filter}'`);
        }
        if (!this._formatters[o.formatter]) {
            throw new Error(`Unknown formatter '${o.formatter}'`);
        }
        if (!this._writers[o.writer]) {
            throw new Error(`Unknown writer '${o.writer}'`);
        }
        const FilterFactory = this._filters[o.filter];
        const Formatter = this._formatters[o.formatter];
        const Writer = this._writers[o.writer];
        const logHandler = new index_1.LogHandler(o.name, FilterFactory.create(), Formatter.create(), Writer.create());
        l.registerHandler(logHandler);
    }
    createEmptyLogger() {
        return new index_1.Logger(index_1.LogRecord, index_1.LoggingInterface);
    }
    createDefaultLogger() {
        const l = new index_1.Logger(index_1.LogRecord, index_1.LoggingInterface);
        this.applyLogHandler(l, {
            name: 'any_to_console_as_single_line',
            filter: 'FilterAny',
            formatter: 'FormatterSingleLine',
            writer: 'WriterToConsole'
        });
        return l;
    }
    createLogger(logHandlers) {
        const l = new index_1.Logger(index_1.LogRecord, index_1.LoggingInterface);
        logHandlers.forEach(handlerOptions => this.applyLogHandler(l, handlerOptions));
        return l;
    }
    registerFilter(entity) {
        this._filters[entity.entityName] = entity;
    }
    registerFormatter(entity) {
        this._formatters[entity.entityName] = entity;
    }
    registerWriter(entity) {
        this._writers[entity.entityName] = entity;
    }
}
exports.LoggerFactory = LoggerFactory;
